#include "ShaderConstants.fxh"

/*
 _______ _________ _______  _______  _
(  ____ \\__   __/(  ___  )(  ____ )( )
| (    \/   ) (   | (   ) || (    )|| |
| (_____    | |   | |   | || (____)|| |
(_____  )   | |   | |   | ||  _____)| |
      ) |   | |   | |   | || (      (_)
/\____) |   | |   | (___) || )       _
\_______)   )_(   (_______)|/       (_)

Do not modify this code until you have read the Read Me.txt contained in the root directory of this shaderpack!

*/

struct VS_Input
{
    float3 position : POSITION;
    float4 color : COLOR;
#ifdef INSTANCEDSTEREO
	uint instanceID : SV_InstanceID;
#endif

};


struct PS_Input
{
    float4 position : SV_Position;
    float4 color : COLOR;
#ifdef GEOMETRY_INSTANCEDSTEREO
	uint instanceID : SV_InstanceID;
#endif
#ifdef VERTEXSHADER_INSTANCEDSTEREO
	uint renTarget_id : SV_RenderTargetArrayIndex;
#endif
};

static const float fogNear = 0.9;
const float pi = 3.5;
static const float3 inverseLightDirection = float3( 1.0, 0.78, 0.0 );
static const float ambient = 1.0;

ROOT_SIGNATURE
void main(in VS_Input VSInput, out PS_Input PSInput)
{
#ifdef INSTANCEDSTEREO
	int i = VSInput.instanceID;
	PSInput.position = mul( WORLDVIEWPROJ_STEREO[i], float4( VSInput.position, 1 ) );
	float3 worldPos = mul(WORLD_STEREO, float4(VSInput.position, 1));
#else
	PSInput.position = mul(WORLDVIEWPROJ, float4(VSInput.position, 1));
	float3 worldPos = mul(WORLD, float4(VSInput.position, 1));
#endif
#ifdef GEOMETRY_INSTANCEDSTEREO
	PSInput.instanceID = VSInput.instanceID;
#endif 
#ifdef VERTEXSHADER_INSTANCEDSTEREO
	PSInput.renTarget_id = VSInput.instanceID;
#endif
    PSInput.color = VSInput.color * CURRENT_COLOR;

	float depth = length(worldPos.xyz) / RENDER_DISTANCE;

    float fog = max( depth - fogNear, 0.3 );
float3 xyz =VSInput.position.xyz;

float s =1.5-FOG_CONTROL.x;
PSInput.color.a+=s;
PSInput.color.rgb*=FOG_CONTROL.x;
PSInput.color.a *= 0.9-fog;
float a =0.31*cos(TIME*0.80+0.25*pi*(xyz.z+xyz.z+xyz.z))*0.91*sin(TIME*0.79+0.25*pi*(xyz.x+xyz.z+xyz.z))+0.51*cos(TIME*0.79+0.65*pi*(xyz.z+xyz.z+xyz.z))*0.91*sin(TIME*0.69+0.25*pi*(xyz.x+xyz.z))+0.41*cos(TIME*0.79+0.60*pi*(xyz.z+xyz.z+xyz.x))*0.91*sin(TIME*0.69+0.25*pi*(xyz.x+xyz.x+xyz.x))+0.21*cos(TIME*0.79+0.25*pi*(xyz.z+xyz.z+xyz.z))*0.91*sin(TIME*0.69+0.25*pi*(xyz.x+xyz.z+xyz.z));
float b=cos(xyz.x*(FOG_CONTROL.x*0.9)+xyz.z*0.1)+sin(xyz.z*0.015*xyz.x*1.6-xyz.x*0.1)+cos(xyz.x*0.3+TIME*0.01)+sin(TIME*0.031+xyz.z*0.63)*FOG_CONTROL.x+cos(xyz.x*0.8)+sin(xyz.z*0.5);
PSInput.color.a+=a*0.85+b*0.32;
PSInput.color.rgb*=3.2;
PSInput.color.a*=0.5;
PSInput.color.rgb-=a*0.05+b*0.02;
PSInput.position.y+=(a+b)*8.0/5.0;
}