#include "ShaderConstants.fxh"

/*
 _______ _________ _______  _______  _
(  ____ \\__   __/(  ___  )(  ____ )( )
| (    \/   ) (   | (   ) || (    )|| |
| (_____    | |   | |   | || (____)|| |
(_____  )   | |   | |   | ||  _____)| |
      ) |   | |   | |   | || (      (_)
/\____) |   | |   | (___) || )       _
\_______)   )_(   (_______)|/       (_)

Do not modify this code until you have read the Read Me.txt contained in the root directory of this shaderpack!

*/

struct VS_Input
{
    float3 position : POSITION;
    float4 color : COLOR;
#ifdef INSTANCEDSTEREO
	uint instanceID : SV_InstanceID;
#endif
};


struct PS_Input
{
    float4 position : SV_Position;
    float4 color : COLOR;
#ifdef GEOMETRY_INSTANCEDSTEREO
	uint instanceID : SV_InstanceID;
#endif
#ifdef VERTEXSHADER_INSTANCEDSTEREO
	uint renTarget_id : SV_RenderTargetArrayIndex;
#endif
};

ROOT_SIGNATURE
void main(in VS_Input VSInput, out PS_Input PSInput)
{
#ifdef INSTANCEDSTEREO
	int i = VSInput.instanceID;
	PSInput.position = mul( WORLDVIEWPROJ_STEREO[i], float4( VSInput.position, 1 ) );
#else
	PSInput.position = mul(WORLDVIEWPROJ, float4(VSInput.position, 1));
#endif
#ifdef GEOMETRY_INSTANCEDSTEREO
	PSInput.instanceID = VSInput.instanceID;
#endif 
#ifdef VERTEXSHADER_INSTANCEDSTEREO
	PSInput.renTarget_id = VSInput.instanceID;
#endif

float4 sky=CURRENT_COLOR;
float4 fog=FOG_COLOR;
float fog_flag=0.0;
float4 c=VSInput.color;
float3 day = float3(166.0,250.0,255.0)/255.0;
float3 rain = float3(1.25,1.25,1.25);
float3 night = float3(72.0,52.0,117.0)/255.0;

if(FOG_CONTROL.x<0.55 && FOG_CONTROL.x>0.1){
fog_flag = 1.0;
sky.rgb = max(sky.rgb*rain,night);
PSInput.color = lerp(sky,fog,c.r);
}

sky.rgb = max(sky.rgb*day,night);

if(fog_flag == 0.0){
PSInput.color = lerp(sky,fog,c.r);
}
}